<?php
namespace User\Service;



/**
 * Registration, login and forgot password functionality.
 * @author andre.fourie
 */
class Registration extends \User\DataBin\Registration
{

	/**
	 * ExecuteAfter: RegisterFinal.
	 * Register a new dealership and its primary user.
	 * @param array $meta
	 * @param object|null $jobRecord
	 * @param object|null $record
	 * @param \Workspace\Utility\ServiceInputParams $contract
	 * @return array
	 */
	public function RegisterCompany($meta, $jobRecord, $session, \Workspace\Utility\ServiceInputParams $contract)
	{
		$contactService = new \Person\Service\Contact();
		$contactService->setWorkflow(new \Person\Workflow());
		$companyService = new \Company\Service\Company();
		$companyService->setWorkflow(new \Company\Workflow());
		$profileService = new \User\Service\Profile();
		$profileService->setWorkflow(new \User\Workflow());
		//\Utility\Debug::errorLog('create contact', $session->Contact);
		$contact = $contactService->create($session->Contact);
		$session->Company['contact'] = $contact->id;
		if (isset($session->BillingContact))
		{
		//\Utility\Debug::errorLog('create billing contact', $session->BillingContact);
			$billingContact = $contactService->create($session->BillingContact);
			$session->Company['contbillingContactact'] = $billingContact->id;
		}
		//\Utility\Debug::errorLog('create company', $session->Company);
		$company = $companyService->create($session->Company);
		$session->Profile['company'] = $company->id;
		if ($session->Profile['permissions'] < 5 || $session->Profile['permissions'] > 8)
		{
			$session->Profile['permissions'] = 5;
		}
		//\Utility\Debug::errorLog('create profile', $session->Profile);
		$profile = $profileService->create($session->Profile);

		$toCompanyId	= $company->id;
		$toProfileId	= $profile->id;
		$email			= $profile->email;
		$mobile			= null;
		$subject		= null;
		$templateName	= 'activation-pending';

		$params = array();
		$params['firstName']	= $profile->firstName;
		$params['familyName']	= $profile->familyName;
		$params['message'] 		= 'Thank you for registering. Your registration was submitted for approval '
								. 'and you will receive email confirmation once your account has been verified.'
								. 'Please note that if any documents were not uploaded they need to be emailed'
								. ' (info@bid4cars.co.za) or faxed (086 618 5584) before we can approve your '
								. 'account.';

		#-> Send welcome notification to User.
		$oNotify = new \Utility\Comms\Notification();
		$oNotify->sendFromTemplate(
				null, null,
				$toCompanyId, $toProfileId,
				$email, $mobile,
				$subject,
				$templateName,
				$params
		);
		
		
		#-> Send welcome notification to Admin.
		$oNotify = new \Utility\Comms\Notification();
		$oNotify->sendFromTemplate(
				null, null,
				$toCompanyId, $toProfileId,
				'info@bid4cars.co.za', $mobile,
				$subject,
				$templateName,
				$params
		);
	}

}